#ifndef __AGM65G_MAVERICK_H
#define __AGM65G_MAVERICK_H

#include "orbitersdk.h"
#include "uwuwWidgets.h"

/**
 * References: 
 * Fighter Wing, Tom Clancy
 *	Harper Collins, 1996
 * 
 * AIAA 1997-3340 "3-D Grain Design and Ballistic Analysis using the SPP97 Code", 
 * S.S. Dunn and D.E. Coats
 */

const double AGM65G_TOTALMASS = 301.5;
const double AGM65G_WARHEADMASS = 136.4;
const double TX633_ISP = 2190.7692;	//AIAA 1997-3340

const double TX633_BOOSTPHASE_DURATION = 1.5;
//The ratio in chamber pressure between boost and sustainer mode
const double TX633_BS_PC_RATIO = 4.13; //AIAA 1997-3340


const double TX633_SUSTAINER_THRUST = 2400;
const double TX633_BOOST_THRUST = TX633_SUSTAINER_THRUST * TX633_BS_PC_RATIO;
const double TX633_SUSTAINER_BT = 33.5;
const double TX633_BOOSTER_BT = 5.0;
const double TX633_PROPELLANTMASS = TX633_SUSTAINER_THRUST * TX633_SUSTAINER_BT / TX633_ISP + 
									 TX633_BOOST_THRUST * TX633_BOOSTER_BT/TX633_ISP;
const double AGM65G_EMPTYMASS = AGM65G_TOTALMASS - AGM65G_WARHEADMASS - TX633_PROPELLANTMASS;

//The maximum flight time after engine ignition before the range safety system activates
const double AGM65G_RSTIME = 3*(TX633_BOOSTER_BT + TX633_SUSTAINER_BT);


const double AGM65G_SEEKERSPEED = 10 * RAD; //10 /s
const double AGM65G_SEEKERFOV = 5 * RAD; //5
const double AGM65G_SEEKERRES = 5 * RAD/320; //320x240 pixel

const double AGM65G_WINGAREA = 0.692;

const int GROUP_SEEKER = 25;
const int GROUP_SEEKERGIMBAL = 24;



void AGM65LiftFuncVH(double aoa, double M, double Re, double* cl, double *cm, double* cd);


class AGM65G: public uwuwWIDGETCONTAINER
{
	VISHANDLE MyVis;
	int iMeshID;
	char pszTargetName[24];
	char cDebugAnimated;
	uwuwWARHEAD warhead;
	MGROUP_ROTATE* ElevationAnim;
	MGROUP_ROTATE* AzimuthAnim;
	PROPELLANT_HANDLE phTX633;
	THRUSTER_HANDLE thTX633;
	THRUSTER_HANDLE thTX633B;
	ATTACHMENTHANDLE ahDeployer;
	//-----------------------------------------
	//Tracking data of the 10 best targets.
	OBJHANDLE hSObject[10];
	double fSObjectRAz[10];
	double fSObjectREl[10];
	double fSObjectError[10];
	double fSObjectRSize[10];
	bool fTargetAquired;
	//------------------------------------------
	double fBurntime;
	double fFlightTime;
	double fLastSimT;
	OBJHANDLE hTarget;
	double fDistanceTraveled;
	double fLastLat;
	double fLastLong;
	double dYaw;
	
	UINT AnimSeekerE;
	UINT AnimSeekerA;
	//Current Seeker movement mode
	int iSeekerMode;
	//The current step of the seeker around the clock. [0..11]
	int iSeekerStep;
	//Azimuth relative to longitudinal axis
	double fSeekerAzimuth;
	//elevation relative to longitudinal axis
	double fSeekerElevation;
	//--------------------
	double fSkrAnimE;
	double fSkrAnimA;
	////////////////////////////
	double fTPitch;

	//true if the seeker system is activated
	bool fActivated;
	//true if armed
	bool fArmed;
	//True if attached to a vessel.
	bool fAttached;
	//Activate the seeker subsystem and begin seeking
	int ActivateSeeker();
	//Arm the missile
	int Arm();
	//Safe the missile
	int Safe();
	//Move seeker to towards target angles
	void MoveSeekerTo(double fTAz, double fTEl, double fDeltaT);
	//Move the Seeker according to the current Mode
	void MoveSeeker(double fSimT, double fDeltaT);
	//Create the animation of the Seeker
	void DefineAnimations();
	double GetNeutralAOA();
//////////////////////////////////////////////////////////////////
	void ControlBank();
	void ControlPitch();
	void ControlYaw();
	void SimIMU();
	double CalculatePerpendicularT0(VECTOR3 &Dir, VECTOR3 &x);
	VECTOR3 CalculatePointP(VECTOR3 &Dir, VECTOR3 &x);
	bool InSeekerFOV(VECTOR3 &vFOVRel, VECTOR3 &RelPos);
	bool CalculateFOVAngles(VECTOR3 &locPos, double *pfAzimuth, double *pfElevation);
	bool AquireTargets();
	void ClearSeekerList();
	void InsertInSeekerList(OBJHANDLE hObject, double fdAz, double fdEl, double fRSize, double fError);
public:
	AGM65G(OBJHANDLE hVessel, int iFlightModel);
	virtual ~AGM65G();
	void onSetClassCaps(FILEHANDLE cfg);
	virtual void onTimestep(double fSimT, double fDeltaT);
	virtual int SendMessage(const char* pszMessage, void* aMessage);
	//////////////////////////////////////////////////////////////////////
	virtual void clbkAnimate(double fSimT);
	virtual int  clbkConsumeBufferedKey(DWORD dwkey, bool fPressed, char* kstate);
	virtual void clbkLoadStateEx(FILEHANDLE scn, void* vs);
	virtual void clbkSaveState(FILEHANDLE scn);
	
	virtual void clbkVisualCreated(VISHANDLE vis, int iRefCount);
	virtual void clbkVisualDestroyed(VISHANDLE vis, int iRefCount);
};

#endif