#include "uwuwWidgets.h"

uwuwWIDGETCONTAINER::uwuwWIDGETCONTAINER(OBJHANDLE hVessel, int iFlightModel)
: VESSEL2(hVessel, iFlightModel), __widgets()
{
	__widgets.clear();
	fLastSimT = -1;
	fDeltaT = 0;
	fDead = false;
}

/**
 * Check if a collision between two objects occured
 */
void uwuwWIDGETCONTAINER::CheckCollision() {
	double ownsize = GetSize();
	fCollision = false;
	fColldV = 0.0;
	VECTOR3 CollImp = _V(0,0,0);
	for(int i = 0; i<oapiGetVesselCount(); i++) {
		OBJHANDLE hOther = oapiGetVesselByIndex(i);
		if(hOther != GetHandle()) {
			double othersize = oapiGetSize(hOther);

			//VESSEL* pOther = oapiGetVesselInterface(hOther) 
			VECTOR3 rpos;
			oapiGetRelativePos(GetHandle(), hOther, &rpos);
			//Check for collision sphere
			if(length(rpos) < ownsize + othersize) {
				//Calculate relative velocity
				VECTOR3 rvel;
				oapiGetRelativeVel(GetHandle(), hOther, &rvel);
				fCollision = true;
				CollImp = CollImp + rvel * oapiGetMass(hOther);

			}
		}

	}
	if(fCollision) {
		fColldV = length(CollImp)/GetMass();
	}
}

void uwuwWIDGETCONTAINER::CheckGroundContact()
{
	double fBodyAlt = GetAltitude() - fabs(sin(GetPitch()) * (GetSize()+0.5));
	double fWingTipAlt = GetAltitude() - fabs(sin(GetBank()) * (GetSize() + 0.5));
	double fBaseAlt = GetAltitude() - fabs(cos(GetPitch()) * (GetSize() + 0.5));
	if(fBodyAlt<=0 || fWingTipAlt <= 0 || fBaseAlt<=0.0)
	{
		fGroundContact = true;
	} else 
	{
		fGroundContact = false;
	}
	

}

void uwuwWIDGETCONTAINER::Finalize()
{
	//Check if is focussed
	if(oapiGetFocusObject() == GetHandle())
	{
		//Search for a new focus object and move to this
	} 

	
	//sprintf(oapiDebugString(), "%s:FINALIZE", GetName());
	//Remove everything
	ClearThrusterDefinitions();
	ClearPropellantResources();
	ClearMeshes();

	oapiDeleteVessel(GetHandle());
	FILE* pFile = fopen("Debug.log", "at");
	if(pFile)
	{
		fprintf(pFile, "Confirm finalize of %s at %fs.\n",
			GetName(), oapiGetSimTime());
		fclose(pFile);
	}
}

void uwuwWIDGETCONTAINER::FullStop()
{
	VESSELSTATUS2 status;
	status.version = 2;
	status.flag = NULL;
	GetStatusEx(&status);
	status.rvel = _V(0.0, 0.0, 0.0);
	DefSetStateEx(&status);
	//sprintf(oapiDebugString(), "%s:FULL STOP", GetName());

}

void uwuwWIDGETCONTAINER::LogEvent(const char* pszMessage,...)
{
	
	va_list marker;
	va_start(marker, marker);
	FILE* pLog = fopen("Event.log", "at");
	if(pLog)
	{
		fprintf(pLog, "%8.2f\t%s:%s\t", oapiGetSimTime(), GetClassName(), GetName());
		vfprintf(pLog, pszMessage, marker);
		fprintf(pLog, "\n");
		fclose(pLog);
	}
	va_end(marker);
	
}

void uwuwWIDGETCONTAINER::clbkPostStep(double fSimT, double fSimDT, double fMJD)
{
	fDeltaT = fSimDT;
	CheckCollision();
	CheckGroundContact();
	
	if(__widgets.size() > 0)
	{
		for(int i = 0; i<__widgets.size(); i++)
		{
			__widgets[i]->OnTimestep(fSimT, fDeltaT);
			if(fGroundContact)
			{
				__widgets[i]->OnGroundContact();
			}
			if(fCollision) {
				__widgets[i]->onCollision(fColldV);
			}
		}
	}
	onTimestep(fSimT, fDeltaT);
	fLastSimT = fSimT;
	if(fDead)
	{
		Finalize();
	}
}

void uwuwWIDGETCONTAINER::clbkSetStateEx(const void* vs)
{
	DefSetStateEx(vs);
}

void uwuwWIDGETCONTAINER::clbkSetClassCaps(FILEHANDLE cfg)
{
	onSetClassCaps(cfg);
	if(__widgets.size() > 0)
	{
		for(int i = 0; i<__widgets.size(); i++)
		{
			
				__widgets[i]->Create();
		}
	}
	
}

void uwuwWIDGETCONTAINER::onCollision(double fdV) {
}

void uwuwWIDGETCONTAINER::onSetClassCaps(FILEHANDLE cfg)
{
}


void uwuwWIDGETCONTAINER::onTimestep(double fSimT, double fDeltaT)
{
}

void uwuwWIDGETCONTAINER::RegisterWidget(uwuwWIDGET* pWidget) 
{
	__widgets.push_back(pWidget);
}	

void uwuwWIDGETCONTAINER::Terminate()
{
	fDead = true;
}


uwuwWIDGET::uwuwWIDGET(uwuwWIDGETCONTAINER *pParent)
{
	___Parent = pParent;
	___Parent->RegisterWidget(this);
}

void uwuwWIDGET::Create()
{
}

void uwuwWIDGET::onCollision(double fdV) {

}

void uwuwWIDGET::OnGroundContact()
{
}

void uwuwWIDGET::OnLoadState(FILEHANDLE scn)
{
}

void uwuwWIDGET::OnSaveState(FILEHANDLE scn)
{
}

void uwuwWIDGET::OnTimestep(double fSimT, double fDeltaT)
{
}

uwuwWIDGETCONTAINER* uwuwWIDGET::Parent() const
{
	return ___Parent;
}


uwuwWARHEAD::uwuwWARHEAD(uwuwWIDGETCONTAINER *pParent, VECTOR3 &ref, double fMass, double lScale, double wScale)
: uwuwWIDGET(pParent)
{
	fArmed = false;
	fIgnited = false;
	fWarheadMass = fMass;
	wref = ref;
	fLScale = lScale;
	fWScale = wScale;
	phWarhead = NULL;
	for(int i = 0; i<32; i++)
	{
		thExplosion[i] = NULL;
	}
	CreateParticleData();
}

void uwuwWARHEAD::Arm()
{
	fArmed = true;
}

void uwuwWARHEAD::Create()
{
	if(phWarhead == NULL)
	{
		phWarhead = Parent()->CreatePropellantResource(fWarheadMass, fWarheadMass);
	}
		//Calculate Explosion "Force" F = dm * w = M/dt * w
	double F = fWarheadMass * 625;
		//Create Thrusters
	VECTOR3 x = wref + _V(0.0, 0.0, 0.5 * fLScale);
	VECTOR3 r = _V(0.0, 0.0, -1.0);
	if(thExplosion[0] == NULL) 
	{
		thExplosion[0] = Parent()->CreateThruster(x, r, F, phWarhead, 2000);
	}

	if(thExplosion[1] == NULL) 
	{
		x = wref + _V(0.0, 0.0, -0.5 * fLScale);
		r = _V(0.0, 0.0,  1.0);
		
		thExplosion[1] = Parent()->CreateThruster(x, r, F, phWarhead, 2000);
	}

		//6 lines

	for(int i = 0; i<6; i++)
	{
		//First two rings: Warhead semisphere
		VECTOR3 radius0 = _V(sin(i * 60 * RAD), cos(i * 60 * RAD), 0);	
		VECTOR3 iradius0 = _V(-sin(i * 60 * RAD), -cos(i * 60 * RAD), 0);	

		double fRadius = 0.5 * fWScale * cos(30 * RAD);
		x = wref + _V(0.0, 0.0, 0.75 * fLScale) + radius0 * fRadius;
		r = iradius0 * cos(30 * RAD) + _V(0.0, 0.0, 0.5);
		if(thExplosion[2+i] == NULL)
		{
			thExplosion[2+i] = Parent()->CreateThruster(x, r, F, phWarhead, 2000);
		}

		x = wref + _V(0.0, 0.0, -0.75 * fLScale) + radius0 * fRadius;
		r = iradius0 * cos(30 * RAD) - _V(0.0, 0.0, 0.5);
		
		if(thExplosion[8+i] == NULL)
		{
			thExplosion[8+i] = Parent()->CreateThruster(x, r, F, phWarhead, 2000);
		}	
		fRadius = 0.5 * fWScale;
		for(int j = 0; j<3; j++)
		{
			x = wref + _V(0.0, 0.0, -0.5 + 0.5 * j) + radius0;
			r = iradius0;
			if(thExplosion[14+6*j+i] == NULL)
			{
				thExplosion[14+6*j+i] = Parent()->CreateThruster(x, r, F, phWarhead, 2000);
			}
		}
			
	}

	int n = 0;
	for(i = 0; i<32; i++) 
	{
		if(thExplosion[i] != NULL)
		{
			//Add Smoke and Fire
			Parent()->AddExhaustStream(thExplosion[i], &smoke1);
			Parent()->AddExhaustStream(thExplosion[i], &flames1);
			n++;
		}
		
	}

	if(fIgnited)
	{
		FILE* pFile = fopen("Debug.log", "at");
		if(pFile)
		{
			fprintf(pFile, "Ignited Warhead of vessel %s at %fs.\n",
				Parent()->GetName(), oapiGetSimTime());
			fclose(pFile);
		}
		n = 0;	
		for(i = 0; i<32; i++) {
			if(thExplosion[i] != NULL)
			{
		
				//Set thruster level
				Parent()->SetThrusterLevel(thExplosion[i], 1.0);
				n++;
			}
		}
		
	}

	//sprintf(oapiDebugString(), "%s:WARHEAD:CREATE.[%d/32]", Parent()->GetName(), n);

}


void uwuwWARHEAD::CreateParticleData()
{

	double fMaxRadius = sqrt(fWarheadMass * 9.0);
	smoke1.flags = 0;
	smoke1.srcsize = fWScale;
	smoke1.srcrate = 100;
	smoke1.v0 = 120.0;
	smoke1.srcspread = 0.05;
	smoke1.lifetime = 50.0;
	smoke1.growthrate = (fMaxRadius - fWScale)/10.0;
	smoke1.atmslowdown = 1.24;
	smoke1.ltype = PARTICLESTREAMSPEC::DIFFUSE;
	smoke1.levelmap = PARTICLESTREAMSPEC::LVL_PSQRT;
	smoke1.lmin = 0.1;
	smoke1.lmax = 1.0;
	smoke1.atmsmap = PARTICLESTREAMSPEC::ATM_PLOG;
	smoke1.amin = 1e-6;
	smoke1.amax = 1.4;
	
	
	flames1.flags = 0;
	flames1.srcsize = fWScale;
	flames1.srcrate = 100;
	flames1.v0 = 300.0;
	flames1.srcspread = 0.25;
	flames1.lifetime = 4.0;
	flames1.growthrate = 30.0 +(fMaxRadius - fWScale)/4.0;
	flames1.atmslowdown = 0.115;
	flames1.ltype = PARTICLESTREAMSPEC::DIFFUSE;
	flames1.levelmap = PARTICLESTREAMSPEC::LVL_PSQRT;
	flames1.lmin = 0.1;
	flames1.lmax = 1.0;
	flames1.atmsmap = PARTICLESTREAMSPEC::ATM_PLOG;
	flames1.amin = 1e-6;
	flames1.amax = 1.4;
	
	flames1.tex = oapiRegisterExhaustTexture ("contrail2");

}

void uwuwWARHEAD::Ignite()
{
	if(!fIgnited)
	{
		fIgnited = true;
		sprintf(oapiDebugString(), "%s:WARHEAD:IGNITION", Parent()->GetName());
		Create();
		
	}
}

void uwuwWARHEAD::OnCollision(double fdV) {
	if(fdV > 1.0 && fArmed && !fIgnited) {
		Ignite();
	} else if(!fArmed) {
		
		Parent()->Terminate();
	}
}

void uwuwWARHEAD::OnGroundContact()
{
	if(fArmed && !fIgnited) {
		Ignite();
	} else if(!fArmed) {
		Parent()->Terminate();
	}
}

void uwuwWARHEAD::OnLoadState(FILEHANDLE scn)
{
}

void uwuwWARHEAD::OnSaveState(FILEHANDLE scn)
{
}

void uwuwWARHEAD::OnTimestep(double fSimT, double fDeltaT)
{
	if(fIgnited)
	{
		if(Parent()->GetPropellantMass(phWarhead) == 0.0) 
		{
			FILE* pFile = fopen("Debug.log", "at");
			if(pFile)
			{
				fprintf(pFile, "Finalize vessel %s at %fs.\nReason: Warhead fully burned.\n",
					Parent()->GetName(), oapiGetSimTime());
				fclose(pFile);
			}
			Parent()->Terminate();
		}
	}
}


